package alarm;

import java.util.List;
import javax.ejb.EJB;
import javax.ejb.Schedule;
import javax.ejb.Singleton;
import database.Alarm;
import database.Maaler;
import database.MaalerEAO;
import database.TempEAO;
import database.Temperatur;
import database.Alarm.Varsel;
import database.AlarmEAO;
import metoder.EpostSender;
import metoder.Metoder;

/*
 * Denne klassen starter tidsstyrte oppgaver.
 * Den starter metodene som kontrollerer alarmene temperaturalarm og måleralarm.
 */
@Singleton
public class BackgroundJobManager {
	
	@EJB
	AlarmEAO alarmEAO;
	@EJB
	TempEAO tempEAO;
	@EJB
	MaalerEAO maalerEAO;
	List<Maaler>maalerList;
	

	
    @Schedule(dayOfWeek="1", hour="0", minute="0", second="0", persistent=false)
    public void someWeeklyJob() {
    	maalerList=maalerEAO.retrieveAll();
    	resettTemperaturAlarm(maalerList);
    }
    

    //Starter maaleralarm og reaktiverMaaler
    @Schedule(hour="*", minute="*/5", second="0", persistent=false)
    public void someQuarterlyJob() {
        // Do your job here which should run every 5 minute of hour.
    	maalerList=maalerEAO.retrieveAll();
    	reaktiverMaaler(maalerList);
    	maalerAlarm(maalerList);
    }

    
    //Starter temperaturalarm
    @Schedule(hour="*", minute="*/1", second="0", persistent=false)
    public void someTemperaturAlarmJob() {
    	maalerList=maalerEAO.retrieveAll();
    	temperaturAlarm(maalerList);
    }
    
    
    //METODER******************************************************************************************
    
	public void reaktiverMaaler(List<Maaler> maalerListe) {
		
		for (Maaler maaler : maalerListe) {
				Temperatur temp = tempEAO.retrieveLast(maaler.getMaalernavn());
				if(temp!=null) {
								//eks kl 12.00.00 > (12.01.00 - )
					if(temp.getTidspunkt().getTime() > (System.currentTimeMillis() - 300000)) {
						maaler.setAktiv(true);
						maalerEAO.update(maaler);
					}
				}
			}//for
	}//reaktiverMaaler
	
	public void maalerAlarm(List<Maaler> maalerListe) {
		
		int disconnectInterval = 600000 * 6; // Antall millisekund uten aktivitet, og maalerern blir satt til passiv. 600 sek = 10 minutt - 600000* 3 = en halv time
		Temperatur alarmtemp = null;
		
		for (Maaler maaler : maalerListe) {
			alarmtemp = tempEAO.retrieveLast(maaler.getMaalernavn());
			
			if(alarmtemp!=null && maaler.getKritiskvarsling()) {
				
					if(maaler.getAktiv()){
						if(alarmtemp.getTidspunkt().getTime() < (System.currentTimeMillis() - disconnectInterval)){
							EpostSender.sendDeviceDisconnectWarning(maaler.getGatenavn());	//Send email
							//Oppdater maalerstatus
							maaler.setAktiv(false);
							maalerEAO.update(maaler);
							lagreVarsling(Varsel.MAALERVARSEL, alarmtemp); //Lagre varsling i databasen
							}//if
					}//if	
				}//if
		}//for
	}
	
	//Kontrollerer om det skal varsles, og sender mail, dersom nødvendig. Autojusterer temperaturgrensen.
	public void temperaturAlarm(List<Maaler> maalerListe) {
		Temperatur temp=null;
		for (Maaler maaler : maalerListe) {
			if(maaler.getKritiskvarsling()) {  //Varsling
				temp = tempEAO.retrieveLast(maaler.getMaalernavn());
				if(temp!=null) {
					
					if(maaler.getTempgrense() < temp.getTemperatur()) {
						maaler.setTempgrense(maaler.getTempgrense()+1);
						lagreVarsling(Varsel.TEMPERATURVARSEL, temp);
						maalerEAO.update(maaler);
						EpostSender.sendTemperatureWarning(maaler.getGatenavn(), String.valueOf(temp.getTemperatur()));
					 }
				}
			}//kritisk varsling 
		}//for
	}//Temperaturalarm
	
	
	//Resetter temperaturvarslinger som har autojustert til over maxgrense.
	public void resettTemperaturAlarm(List<Maaler> maalerListe) {
		for (Maaler maaler : maalerListe) {
			if(maaler.getTempgrense() > 40) {
				maaler.setTempgrense(40);
				maalerEAO.update(maaler);
			}
		}
	}
	
	
	
	//Lagrer varslinger i databasen
	public void lagreVarsling(Varsel varsel, Temperatur temp ) {
		Metoder metoder=new Metoder();
		Alarm alarm = new Alarm();
		alarm.setMaalernavn(temp.getMaalerNavn());
		alarm.setTid(metoder.getTid());
		alarm.setDato(metoder.getDato());
		alarm.setVarsel(varsel);
		alarmEAO.create(alarm);
	}
}
