package database;

import java.util.Date;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;

import metoder.Metoder;


/*
 * Denne klassen kommuniserer til databasen. Den lagrer, sletter og henter informasjon i databasen.
 */

@Stateless
public class TempEAO {

	Metoder methods=new Metoder();
	
	@PersistenceContext(name = "TemperaturPercistenceUnit")
    private EntityManager em;


	synchronized public void create(Temperatur a) {
			em.persist(a);
	}


	synchronized public void delete(Temperatur a) {
			em.remove(em.merge(a));
	}
		
	//Finner primær id
	synchronized public Temperatur retrieveOne(Integer id) {
			return  em.find(Temperatur.class, id);
	}
	
	//Henter hele databasen
	synchronized public List<Temperatur> retrieveAlle() {
		List<Temperatur> liste = null;
		
		TypedQuery<Temperatur> query = em.createQuery("SELECT a FROM Temperatur a", Temperatur.class);
		liste = query.getResultList();
		
		return liste;
	}
	
	
	//Henter den temperaturen som ble sist lagret i databasen. 
	synchronized public Temperatur retrieveLast(String arduinonavn) {
		Temperatur temp=new Temperatur();
		String te="";
		
	    TypedQuery<Temperatur> q= em.createQuery("SELECT MAX(e.id) FROM Temperatur e WHERE e.maalernavn = :maalernavn", Temperatur.class);  
	    q.setParameter("maalernavn", arduinonavn);
	  
	    List<Temperatur>liste=q.getResultList();
	    if(liste.equals("[null]")){ //Denne burde vært fikset, men det er like før deadline, og jeg tør ikke gjøre noen forandringer nå. Det har ikke gitt feilmeldinger i prosjektet. Kun Eclipse som klager.
	    	return temp;
	    } 
	    else {
	    	te=liste.toString();
	    	te=methods.charReplace(te); //tar vekk klammer
	    	int id=Integer.parseInt(te);
	  	    temp=retrieveOne(id);//henter objektet
	    }
		return temp;
}


	//Henter alle med ett navn
	synchronized public List<Temperatur> retrieveTermometer(String kortNavn) {
		List<Temperatur> liste = null;
		
		TypedQuery<Temperatur> query = em.createQuery("SELECT a FROM Temperatur a WHERE a.maalernavn = :navn", Temperatur.class);
		query.setParameter("navn", kortNavn);
		liste = query.getResultList();
		
		return liste;
	}  
	
		
	//AVG temperaturer*************************************************************************************
	synchronized public String avgTemp(Temperatur temperatur, Date d) {
		String te="";
		//Finner avg temperatur
		TypedQuery<Temperatur> q= em.createQuery("SELECT AVG(e.temperatur) FROM Temperatur e WHERE e.maalernavn = :navn AND e.dato = :dato", Temperatur.class); 
		q.setParameter("navn", temperatur.getMaalerNavn());;
	    q.setParameter("dato", d);
	    List<Temperatur>liste=q.getResultList();	
		te=methods.utskrift(liste.toString());
	    return te;
	}

	

	synchronized public String avgTempBetween(Temperatur temperatur, Date d, Date e) {
		String te="";
		//Finner avg temperatur
		TypedQuery<Temperatur> q= em.createQuery("SELECT AVG(e.temperatur) FROM Temperatur e WHERE e.maalernavn = :navn AND e.dato BETWEEN :startdato AND :sluttdato", Temperatur.class); 
		q.setParameter("navn", temperatur.getMaalerNavn());
	    q.setParameter("startdato", d);
	    q.setParameter("sluttdato", e);
	    List<Temperatur>liste=q.getResultList();
		te=methods.utskrift(liste.toString());
	    return te;
	}

//Min temperaturer
	synchronized public String minTemp(Temperatur temperatur, Date d) {
		String te="";
		//Finner max temperatur
		TypedQuery<Temperatur> q= em.createQuery("SELECT MIN(e.temperatur) FROM Temperatur e WHERE e.maalernavn = :navn AND e.dato = :dato", Temperatur.class); 
		q.setParameter("navn", temperatur.getMaalerNavn());;
	    q.setParameter("dato", d);
	    List<Temperatur>liste=q.getResultList();
		te=methods.utskrift(liste.toString());
	    return te;
	}
	
	synchronized public String minTempBetween(Temperatur temperatur, Date d, Date e) {
		String te="";
		//Finner avg temperatur
		TypedQuery<Temperatur> q= em.createQuery("SELECT MIN(e.temperatur) FROM Temperatur e WHERE e.maalernavn = :navn AND e.dato BETWEEN :startdato AND :sluttdato", Temperatur.class); 
		q.setParameter("navn", temperatur.getMaalerNavn());
	    q.setParameter("startdato", d);
	    q.setParameter("sluttdato", e);
	    List<Temperatur>liste=q.getResultList();
		te=methods.utskrift(liste.toString());
	    return te;
	}

	

	
	//Max temperaturer*************************************************************************************
	synchronized public String maxTemp(Temperatur temperatur, Date d) {
		String te="";
		//Finner max temperatur
		TypedQuery<Temperatur> q= em.createQuery("SELECT MAX(e.temperatur) FROM Temperatur e WHERE e.maalernavn = :navn AND e.dato = :dato", Temperatur.class); 
		q.setParameter("navn", temperatur.getMaalerNavn());;
	    q.setParameter("dato", d);
	    List<Temperatur>liste=q.getResultList();
		te=methods.utskrift(liste.toString());
	    return te;
	}
	
	synchronized public String maxTempBetween(Temperatur temperatur, Date d, Date e) {
		String te="";
		//Finner avg temperatur
		TypedQuery<Temperatur> q= em.createQuery("SELECT MAX(e.temperatur) FROM Temperatur e WHERE e.maalernavn = :navn AND e.dato BETWEEN :startdato AND :sluttdato", Temperatur.class); 
		q.setParameter("navn", temperatur.getMaalerNavn());
	    q.setParameter("startdato", d);
	    q.setParameter("sluttdato", e);
	    List<Temperatur>liste=q.getResultList();
		te=methods.utskrift(liste.toString());
	    return te;
	}

}// class

