package servlets;

import java.io.IOException;
import java.util.List;
import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import database.Maaler;
import database.MaalerEAO;
import database.TempEAO;
import database.Temperatur;
import database.AlarmEAO;
import metoder.Metoder;

/*
 * Denne klassen tar imot requester i fra arduinomålerene og lagrer dem 
 * i ett temperaturobjekt. Tilfører id, tid, dato og tidspunkt, og lagrer det i databasen
 */
@WebServlet("/ArduinoServlet")
public class ArduinoServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;

	@EJB
	TempEAO tempEAO;
	@EJB
	MaalerEAO maalerEAO;
	@EJB
	AlarmEAO alarmEAO;
	List<Maaler> maalerListe;
	String arduinonavn;
	double temp;

	public ArduinoServlet() {
		super();
	}

	protected void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {

	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		temp = 0;
		arduinonavn = "no name";
		Metoder metoder = new Metoder();
		boolean navnKontroll=false;
		boolean tempKontroll=false;

		if (request.getParameter("navn") != null) {
			arduinonavn = request.getParameter("navn");	
			maalerListe=maalerEAO.retrieveAll(); 	//Test om arduinonavn finnes i databasen
			for(Maaler m: maalerListe) {
				if(m.getMaalernavn().equals(arduinonavn)) {
					navnKontroll=true;
					break;
				}
			}				
		}
	
		if (request.getParameter("temp") != null) {
			String verdi=request.getParameter("temp");
	         try{    
	                temp=Double.parseDouble(verdi);  
	                if(temp<100) { //for å begrense skaden ved en eventuell "Denial of service attack".
	                	 tempKontroll=true;
	                }
	            }catch(NumberFormatException fe){
	                //System.out.println("Invalid: " + verdi);
	            }
		}//if
		
		if(navnKontroll && tempKontroll) {
			Temperatur temperatur = new Temperatur();
			temperatur.setTemperatur(temp);
			temperatur.setMaalerNavn(arduinonavn);
			temperatur.setDato(metoder.getDato());
			temperatur.setTid(metoder.getTid());
			temperatur.setTidspunkt(metoder.getTidspunkt());
			tempEAO.create(temperatur);
		}
	}// doPost
}