package webServlets;

import java.io.IOException;
import java.util.List;

import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import database.Bruker;
import database.BrukerEAO;
import sesjon.PassordUtil;
import sesjon.ValiderInput;

/**
 * Oppretter bruker og sender tråden videre til nybrukerbekreftelse
 */
@WebServlet("/BrukerServlet")
public class BrukerServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
	@EJB
	BrukerEAO brukerEAO;
	String utlogging = "";
	Bruker nybruker;
	
    public BrukerServlet() {
        super();
    }
    
	public void init() {
		utlogging = getServletConfig().getInitParameter("tidParameter");
	}


	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		request.setCharacterEncoding("UTF-8");
	
        HttpSession sesjon = request.getSession(false);  //SESJON Returnerer null eller eksisterende sesjon
        if (sesjon == null || sesjon.getAttribute("Bruker") == null) {
        	request.setAttribute("feilmelding", "Ugyldig sesjon");
            response.sendRedirect("index.jsp");
            return;
        }
		
        
		if(request.getParameter("nyregistrertbruker")==null) {
			request.getRequestDispatcher("WEB-INF/nybruker.jsp").forward(request, response);
			return;
		}
		
        request.setAttribute("nybruker", nybruker);
		request.getRequestDispatcher("WEB-INF/nybrukerbekreftelse.jsp").forward(request, response);
	}


	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		
		ValiderInput validerInput=new ValiderInput();
		PassordUtil passordUtil=new PassordUtil();

		String fornavn=request.getParameter("fornavn");
		fornavn= validerInput.escapeHtml(fornavn);
		String etternavn=request.getParameter("etternavn");
		etternavn= validerInput.escapeHtml(etternavn);
		String mob=request.getParameter("mobil");
		mob= validerInput.escapeHtml(mob);
		String epost=request.getParameter("epost");
		epost= validerInput.escapeHtml(epost);
		String passord=request.getParameter("passord");
		passord= validerInput.escapeHtml(passord);
		String passordRep=request.getParameter("passordRepetert");
		passordRep= validerInput.escapeHtml(passordRep);
		
		
		//Test om bruker finnes i databasen fra før
		int mobil=0;
		boolean testMobil=validerInput.validerMobil(mob);
		if(testMobil) {
			mobil=Integer.parseInt(mob);
			testMobil=finnBruker(mobil);
		}
		
		
		boolean testFornavn=validerInput.validerForNavn(fornavn);
		boolean testEtternavn=validerInput.validerEtterNavn(etternavn);
		boolean testEpost=validerInput.validerEpost(epost);
		boolean testPassord=validerInput.validerPassord(passord);
		boolean testPassordRepeter=validerInput.validerPassordRepeter(passord, passordRep, testPassord);
		
		if(!testFornavn || !testEtternavn || !testMobil || !testEpost ||!testPassord || !testPassordRepeter) {
			request=validerInput.settFeilmeldinger(request, testFornavn, testEtternavn, testMobil, testEpost, testPassord, testPassordRepeter,
					fornavn, etternavn, mob, epost, passord, passordRep);
			request.getRequestDispatcher("WEB-INF/nybruker.jsp").forward(request, response);
			return;
		}	
		
		//Opprett ny bruker
	  	String kryptertPassord = passordUtil.krypterPassord(passord);
		nybruker=new Bruker(fornavn, etternavn, mobil, epost, kryptertPassord); 
		brukerEAO.create(nybruker); 
		
		response.sendRedirect("BrukerServlet?nyregistrertbruker");
	}
	
	//finner deltaker i databasen.
	public boolean finnBruker(int mobil) {
		boolean test=true;
		List<Bruker>liste = brukerEAO.retrieveAlle();
		for(Bruker a : liste) {
		  if(a.getMobil()==mobil){
			  test=false;	//mobil finnes i databasen og skal ikke lagres paa nytt
		  }
	  }
		return test;
	}

}
