package webServlets;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import database.Maaler;
import database.MaalerEAO;
import database.TempEAO;
import database.Temperatur;
import metoder.MovingMeasurement;

/*
 *DoGet klargjør for visning på oversikt.jsp. Aller målerer blir hentet
 *fra databasen. Deretter hentes det siste lagrede temperaturobjektet til hver måler. 
 *Så blir infoen samlet i objektet MovingMeasurement, og overført til oversikt.jsp, og kjørt i en loop.  
 */
@WebServlet("/OversiktServlet")
public class OversiktServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	@EJB
	TempEAO tempEAO;
	@EJB
	MaalerEAO navnEAO;
	List<Maaler>navnListe;
	List<MovingMeasurement>jspliste;
      
	@Override
	public void init() throws ServletException {
		navnListe=null;
		navnListe=navnEAO.retrieveAll();
	}

    public OversiktServlet() {
        super();
    }

    
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		HttpSession sesjon = request.getSession(false);  //SESJON Returnerer null eller eksisterende sesjon
        if (sesjon == null || sesjon.getAttribute("Bruker") == null) {
        	request.setAttribute("feilmelding", "Sesjon ugyldig");
        	request.getRequestDispatcher("index.jsp").forward(request,response);
        }
        
        else {
			jspliste=new ArrayList<MovingMeasurement>();
			
			if(navnListe==null ){	//Test om databasen er tom
				response.sendRedirect("LogoutServlet");
			}     
			
			for (Maaler navn: navnListe) {
			
				Temperatur temperatur=new Temperatur();
				temperatur= tempEAO.retrieveLast(navn.getMaalernavn());
				
				if(temperatur==null) {
					//Ikke gj�r noe, dersom det ikke finnes en temperatur
				}
				
				else {		
					MovingMeasurement mm=new MovingMeasurement();
					mm.setCurrentTemp(temperatur.utskriftTemperatur());
					mm.setMaalernavn(navn.getMaalernavn());
					mm.setGatenavn(navn.getGatenavn());
					mm.SetDato(temperatur.utskriftDato());
					mm.setTid(temperatur.utskriftTid());
					jspliste.add(mm);
				}
	
			}//for
			
			request.setAttribute("jspliste", jspliste);
			request.getRequestDispatcher("WEB-INF/oversikt.jsp").forward(request, response);
        }//else
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.sendRedirect("OversiktServlet");
	}
}
