package webServlets;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import database.Maaler;
import database.MaalerEAO;
import database.TempEAO;
import database.Temperatur;
import metoder.Metoder;
import metoder.MovingTemperatures;

/*
 * For å nå denne servleten så starter bruker på oversikt.jsp. Der velger bruker en måler og blir sendt til doPost.
 * DoPost - Der blir valgt måler hentet i databasen
 * 
 * DoGet
 * Her blir det nå samlet statistiske opplysninger om valgt måler. Det blir gjort ved å starte spørringer mot databasen.
 * Resultatene blir lagret i objektene MovingTemperatures. Deretter blir en array med objektene sendt til utskrift på
 * temperaturmaaler.jsp.
 * 
 * Ved å overfør alle variabler til String før utskrift(MovingTemperatures), 
 * så fikk prosjektet vekk alle feilmeldinger om nullpointer exception, som typed query lagde.
 * 
 */
@WebServlet("/TemperaturmaalerServlet")
public class TemperaturmaalerServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
	@EJB
	TempEAO tempEAO;
	@EJB
	MaalerEAO maalerEAO;
	Maaler maaler;
	Maaler n;
	int sisteLagredeTemperatur=0;
	String melding="";
	String jsp="";
	String gatenavn = "";
	String arduinonavn = "";
	Metoder metoder;
	
	@Override
	public void init() throws ServletException {
		
	}
	
    public TemperaturmaalerServlet() {
        super();
    }
    
    
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		HttpSession sesjon = request.getSession(false);  //SESJON Returnerer null eller eksisterende sesjon
        if (sesjon == null || sesjon.getAttribute("Bruker") == null) {
        	request.setAttribute("feilmelding", "Sesjon ugyldig");
        	request.getRequestDispatcher("index.jsp").forward(request,response);
        }
        else {
			Temperatur temperatur=new Temperatur();
			metoder=new Metoder();
			List<MovingTemperatures>liste=new ArrayList<MovingTemperatures>();
			maaler=new Maaler();
			maaler = (Maaler)getServletContext().getAttribute("navn");
			
			temperatur= tempEAO.retrieveLast(maaler.getMaalernavn());
			if(temperatur==null) {
				//Ikke gj�r noe, dersom det ikke finnes en temperatur, //Eventuelt lag en feilmelding
			}
			else {
				MovingTemperatures min=new MovingTemperatures();
				min=hentMin(temperatur);
				
				MovingTemperatures max=new MovingTemperatures();
				max=hentMax(temperatur);
				
				MovingTemperatures avg=new MovingTemperatures();
				avg=hentAvg(temperatur);
			
				//Utskrift på jsp
				request.setAttribute("navnPaaMaaler", maaler.getGatenavn());
				request.setAttribute("temperaturObjekt", temperatur);
				request.setAttribute("sisteTemperatur",  temperatur.utskriftTemperatur()); //temperature in real time
				
				liste.add(max);
				liste.add(avg);
				liste.add(min);
				
				request.setAttribute("temperaturer", liste);
			}//else
			
			request.getRequestDispatcher("WEB-INF/temperaturmaaler.jsp").forward(request, response);
        }//else	
	}//doGet


	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		HttpSession sesjon = request.getSession(false);  //SESJON Returnerer null eller eksisterende sesjon
        if (sesjon == null || sesjon.getAttribute("Bruker") == null) {
        	request.setAttribute("feilmelding", "Sesjon ugyldig");
        	request.getRequestDispatcher("index.jsp").forward(request,response);
        }
        else {
		
			if(request.getParameter("temp") != null) {
				arduinonavn=request.getParameter("temp").toString();//gatenavn
				n=new Maaler();
				n=maalerEAO.retrieveOne(arduinonavn);
				getServletContext().setAttribute("navn", n);	
			}
			response.sendRedirect("TemperaturmaalerServlet");
        }//else	
	}//doPost
	
	
	public MovingTemperatures hentMin(Temperatur temperatur) {
		//Henter query sporringer fra databasen.
		//Henter mintemperatur
		MovingTemperatures min=new MovingTemperatures("min", metoder.getDato(), metoder.oneDayAgo(), metoder.twoDaysAgo());
		min.setTempToday(tempEAO.minTemp(temperatur, metoder.getDato()) );
		min.setTempOneDayAgo(tempEAO.minTemp(temperatur, metoder.oneDayAgo()));
		min.setTempTwoDaysAgo(tempEAO.minTemp(temperatur, metoder.twoDaysAgo()));
		min.setTempLastSevenDays(tempEAO.minTempBetween(temperatur, metoder.sevenDaysAgo(), metoder.getDato()));
		min.setTempLastThirtyDays(tempEAO.minTempBetween(temperatur, metoder.oneMonthAgo(), metoder.getDato()));
		return min;
	}
	
	public MovingTemperatures hentMax(Temperatur temperatur) {
		MovingTemperatures max=new MovingTemperatures("max", metoder.getDato(), metoder.oneDayAgo(), metoder.twoDaysAgo());
		//Henter maxtemperatur
		max.setTempToday(tempEAO.maxTemp(temperatur, metoder.getDato()) );
		max.setTempOneDayAgo(tempEAO.maxTemp(temperatur, metoder.oneDayAgo()));
		max.setTempTwoDaysAgo(tempEAO.maxTemp(temperatur, metoder.twoDaysAgo()));
		max.setTempLastSevenDays(tempEAO.maxTempBetween(temperatur, metoder.sevenDaysAgo(), metoder.getDato()));
		max.setTempLastThirtyDays(tempEAO.maxTempBetween(temperatur, metoder.oneMonthAgo(), metoder.getDato()));
		return max;
	}
	
	public MovingTemperatures hentAvg(Temperatur temperatur) {
		MovingTemperatures avg=new MovingTemperatures("avg",metoder.getDato(), metoder.oneDayAgo(), metoder.twoDaysAgo());
		//Henter gj.temperatur		
		avg.setTempToday(tempEAO.avgTemp(temperatur, metoder.getDato()) );
		avg.setTempOneDayAgo(tempEAO.avgTemp(temperatur, metoder.oneDayAgo()));
		avg.setTempTwoDaysAgo(tempEAO.avgTemp(temperatur, metoder.twoDaysAgo()));
		avg.setTempLastSevenDays(tempEAO.avgTempBetween(temperatur, metoder.sevenDaysAgo(), metoder.getDato()));
		avg.setTempLastThirtyDays(tempEAO.avgTempBetween(temperatur, metoder.oneMonthAgo(), metoder.getDato()));
		return avg;
	}
}//Class
